# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv

from six import StringIO

from driverng import MessageV2, Worklist
from driverng.splitted import Multiple, Splitted, String


class AtilaPowerGeneResultsBase(Splitted):

    SEPARATOR = "\t"


class AtilaPowerGeneResult(object):
    FIELDS = [
        String("well"),
        String("property"),
        String("sample_id"),
        String("sample_name"),
        String("std_con"),
        String("target"),
        String("dye"),
        Multiple("results", (String("resu"),), separator="\t"),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for index, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[index:])
                break
            else:
                setattr(self, field.name, line[index])


class AtilaPowerGeneResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: AtilaPowerGeneResult}

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str("\t"))
        for line in reader:
            if not len(line):
                continue
            if len(line) == 57:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "atila_powergene", "label": "ATILA POWERGENE"}

    def parse(self, data):
        obj = AtilaPowerGeneResultsMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        RESULTS_CODES = ()
        for m in message:
            if m.well.strip() == "Well" and m.property.strip() == "Property":
                RESULTS_CODES = m.results
                continue
            if RESULTS_CODES:
                if m.target == "controle interne":
                    sample = worklist.get_or_create(
                        control_lot=m.sample_id, sample_id=m.sample_id, type="control"
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=m.sample_id, rack=m.well, type="patient"
                    )
                for index, result in enumerate(m.results):
                    if RESULTS_CODES[index] and result:
                        sample.results.add(
                            code=RESULTS_CODES[index].strip(), value=result
                        )
        return worklist
