# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import datetime

from six import StringIO

from driverng import MessageV2, Worklist
from driverng.splitted import Multiple, Splitted, String


class ExiclerResultsBase(Splitted):

    SEPARATOR = ","


class ExiclerResult(object):
    FIELDS = [
        String("well"),
        String("sample_name"),
        String("probe_name"),
        String("type"),
        String("flu_dye"),
        String("quencher"),
        String("ct"),
        Multiple("results", (String("resu"),), separator=","),
    ]

    def __init__(self, **kwargs):
        self.instrument = kwargs.pop("instrument")
        self.last_analysis = kwargs.pop("last_analysis")

    def parse(self, line):
        fields = self.FIELDS
        for index, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[index:])
                break
            else:
                setattr(self, field.name, line[index])


class ExiclerResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: ExiclerResult}

    instrument = None

    def add(self, type_message=None, **kwargs):
        kwargs["instrument"] = self.instrument
        kwargs["last_analysis"] = self.last_analysis
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str(","))
        for line in reader:
            if not len(line):
                continue
            if len(line) > 5:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)
            elif line[0].startswith("Instrument"):
                liste_instrument = line[0].split(":")
                if len(liste_instrument) > 1:
                    self.instrument = liste_instrument[1].strip()
            elif line[0].startswith("Last Analysis") and len(line) == 2:
                self.last_analysis = line[1].strip()


class Driver(MessageV2):
    SPEC = {"code": "exicycler", "label": "Exicycler ORGENTEC"}

    def parse(self, data):
        obj = ExiclerResultsMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        RESULTS_CODES = ()
        for m in message:
            if m.well.strip() == "Well" and m.sample_name.strip() == "Sample Name":
                RESULTS_CODES = m.results
                continue
            if RESULTS_CODES:
                sample = worklist.get_or_create(
                    sample_id=m.sample_name, rack=m.well, type="patient"
                )
                for index, result in enumerate(m.results):
                    if result == "Auto":
                        continue
                    if RESULTS_CODES[index] and result:
                        sample.results.add(
                            code="{}{}".format(RESULTS_CODES[index].strip(), m.flu_dye),
                            value=result,
                            instrument=m.instrument,
                            date=datetime.datetime.strptime(
                                m.last_analysis, "%b %d %Y %H:%M:%S"
                            ),
                        )

        return worklist
